@@ -5,6 +5,7 @@ import logging
 from io import BytesIO
 from datetime import datetime
 import json
+import traceback
 
 from urllib.request import urlopen
 from email.mime.text import MIMEText
@@ -35,10 +36,19 @@ from .renderer.excel import ExcelRenderer
 storage_name = os.getenv('STORAGE_NAME')
 
 global ACCESS_KEY_ID, SECRET_ACCESS_KEY, REGION_NAME, BUCKET_NAME
-ACCESS_KEY_ID = os.getenv('ACCESS_KEY_ID', '')
-SECRET_ACCESS_KEY = os.getenv('SECRET_ACCESS_KEY', '')
-REGION_NAME = os.getenv('REGION_NAME', '')
-BUCKET_NAME = os.getenv('BUCKET_NAME', '')
+
+if storage_name == 's3':
+    ACCESS_KEY_ID = os.getenv('ACCESS_KEY_ID')
+    SECRET_ACCESS_KEY = os.getenv('SECRET_ACCESS_KEY')
+    REGION_NAME = os.getenv('REGION_NAME')
+    BUCKET_NAME = os.getenv('BUCKET_NAME')
+elif storage_name == 'ceph':
+    ACCESS_KEY_ID = os.getenv('ACCESS_KEY_ID')
+    SECRET_ACCESS_KEY = os.getenv('SECRET_ACCESS_KEY')
+    BUCKET_NAME = os.getenv('BUCKET_NAME')
+    ENDPOINT_URL = os.getenv('ENDPOINT_URL')
+else:
+    logger.info("No storages are found")
 
 
 def get_token():
@@ -166,19 +176,41 @@ def get_epoc_from_datetime_string(str_datetime):
 
 def get_result_by_run(run_id, field=None, default_value=None):
     try:
-        s3 = get_s3_client()
         run_id= f'{PLATFORM_ROUTE}/{run_id}/json/{run_id}'
-        res_object = s3.get_object(Bucket=BUCKET_NAME,Key=run_id)
-        serializedObject = res_object['Body'].read()
-        result = json.loads(serializedObject)
-        if field:
-            result = result.get(field, default_value)
-        return result
+        if storage_name == 's3':
+            s3 = get_s3_client()
+            res_object = s3.get_object(Bucket=BUCKET_NAME,Key=run_id)
+            serializedObject = res_object['Body'].read()
+            result = json.loads(serializedObject)
+            if field:
+                result = result.get(field, default_value)
+            return result
+        elif storage_name == 'ceph':
+            s3 = get_s3_ceph()
+            data = BytesIO()
+            res_object = s3.Bucket(BUCKET_NAME).download_fileobj(Fileobj=data,Key=run_id)   
+            res_object = data.getvalue()
+            result = json.loads(res_object)
+            if field:
+                result = result.get(field, default_value)
+            return result
+        else:
+            logger.info("No storages are found")
     except Exception:
         logger.error('An error occurred (NoSuchKey) when calling the GetObject operation: The specified key does not exist')
         pass
 
 
+def get_response(url, type, params=None):
+    start_time = int(time.time())
+    logging.info(f'api type: {type}, : url : {url}')
+    res = call_get_requests(url, params=None, verify=True)
+    duration = int(time.time()) - start_time
+    if duration > API_TIME_STACKS:
+        logging.info(f'Get {type} API response took %d (greater than %d) seconds, url : {url}', duration, API_TIME_STACKS)
+    return res
+
+
 def get_ses_client():
     return boto3.client('ses',
                         region_name=REGION_NAME,
@@ -193,6 +225,13 @@ def get_s3_client():
                         aws_secret_access_key=SECRET_ACCESS_KEY)
     
 
+def get_s3_ceph():
+    return boto3.resource('s3',
+                            endpoint_url=ENDPOINT_URL,
+                            aws_access_key_id=ACCESS_KEY_ID,
+                            aws_secret_access_key=SECRET_ACCESS_KEY)
+
+
 def send_email(subject, from_email, to_emails, body, attachment=None):
     message = MIMEMultipart()
     message['Subject'] = subject
@@ -225,20 +264,35 @@ def upload_to_s3(content, location):
     :param: content: bytes
     :param: location: str
     '''
-    s3 = boto3.resource('s3',
-                        region_name=REGION_NAME,
-                        aws_access_key_id=ACCESS_KEY_ID,
-                        aws_secret_access_key=SECRET_ACCESS_KEY)
-    object_url = f'https://{BUCKET_NAME}.s3.{REGION_NAME}.amazonaws.com/{location}'
-
-    try:
-        s3.Bucket(BUCKET_NAME).put_object(Body=content,
-                                             Key=location)
-        #return object_url
-        return location
-    except Exception:
-        pass
-    
+    if storage_name == 's3':
+        s3 = boto3.resource('s3',
+                            region_name=REGION_NAME,
+                            aws_access_key_id=ACCESS_KEY_ID,
+                            aws_secret_access_key=SECRET_ACCESS_KEY)
+        object_url = f'https://{BUCKET_NAME}.s3.{REGION_NAME}.amazonaws.com/{location}'
+        try:
+            s3.Bucket(BUCKET_NAME).put_object(Body=content,
+                                                 Key=location)
+            #return object_url
+            return location
+        except Exception:
+            pass
+    elif storage_name == 'ceph':
+        try:
+            s3 = boto3.resource('s3',
+                                endpoint_url=ENDPOINT_URL,
+                                aws_access_key_id=ACCESS_KEY_ID,
+                                aws_secret_access_key=SECRET_ACCESS_KEY)
+            s3.Bucket(BUCKET_NAME).put_object(Bucket=BUCKET_NAME,
+                                            Key=location,
+                                            Body=content)
+            #return object_url
+            return location
+        except Exception:
+            pass
+    else:
+        logger.info("No storages are found")
+        
 
 def generate_pdf(analysis_run):
     logger.info('Entered into pdf generation process')
@@ -265,9 +319,9 @@ def generate_pdf(analysis_run):
 
 
 
-def generate_excel(analysis_run, orgId, integrationId, excel_data, report_gen_start_time, report_gen_completed_time, file_name=None):
+def generate_excel(analysis_run, orgId, excel_data, report_gen_start_time, report_gen_completed_time, file_name=None):
     try:
-        excel_renderer = ExcelRenderer(analysis_run, orgId, integrationId, excel_data, report_gen_start_time, report_gen_completed_time)
+        excel_renderer = ExcelRenderer(analysis_run, orgId, excel_data, report_gen_start_time, report_gen_completed_time)
         workbook = excel_renderer.render()
     except Exception as ex:
         raise ex
@@ -287,9 +341,9 @@ def diff_sec(st, et):
     return difference
 
 
-def update_status_url(analysisRunId, tenantId, integrationId, genStime, genEtime, status=None):
+def update_status_url(analysisRunId, tenantId, genStime, genEtime, status=None):
 
-    url = BASE_API_URL + f'/analytics/api/v7/tenants/{tenantId}/{integrationId}/runs/{analysisRunId}'
+    url = BASE_API_URL + f'/analytics/api/v3/tenants/{tenantId}/runs/{analysisRunId}'
     data={
             "status" : status,
             "runDurStartDate" : genStime,
@@ -305,15 +359,24 @@ def update_status_url(analysisRunId, tenantId, integrationId, genStime, genEtime
     logger.info('Status update response is %s', res)
 
 
-def update_results_url(gen_start_time, analysisRunId, tenantId, integrationId, json_result_url=None, gen_completed_time=None, excel_result_url=None, pdf_result_url=None, failure_reason=None, status=None):
+def update_results_url(gen_start_time, analysisRunId, tenantId, json_result_url=None, gen_completed_time=None, excel_result_url=None, pdf_result_url=None, failure_reason=None, status=None):
 
-    url = BASE_API_URL + f'/analytics/api/v7/tenants/{tenantId}/{integrationId}/runs/{analysisRunId}'
+    url = BASE_API_URL + f'/analytics/api/v3/tenants/{tenantId}/runs/{analysisRunId}'
     
     if pdf_result_url == None:
         data={
                 "status" : status,
                 "resultUrl" : json_result_url,
-                "filePath" : excel_result_url,
+                "xlsxFilePath" : excel_result_url,
+                "repGenStartTime" : gen_start_time,
+                "repGenEndTime" : gen_completed_time,
+                "failureReason" : failure_reason
+        }
+    elif excel_result_url == None:
+        data={
+                "status" : status,
+                "resultUrl" : json_result_url,
+                "pdfFilePath" : pdf_result_url,
                 "repGenStartTime" : gen_start_time,
                 "repGenEndTime" : gen_completed_time,
                 "failureReason" : failure_reason
@@ -322,7 +385,8 @@ def update_results_url(gen_start_time, analysisRunId, tenantId, integrationId, j
         data={
                 "status" : status,
                 "resultUrl" : json_result_url,
-                "filePath" : pdf_result_url,
+                "pdfFilePath" : pdf_result_url,
+                "xlsxFilePath" : excel_result_url,
                 "repGenStartTime" : gen_start_time,
                 "repGenEndTime" : gen_completed_time,
                 "failureReason" : failure_reason
@@ -355,6 +419,22 @@ def upload_excel_s3(local_file, bucket, s3_file):
         return False
 
 
+#Upload excel file to ceph
+def upload_excel_ceph(local_file, bucket, s3_file):
+    try:
+        s3 = get_s3_ceph()
+        s3.Bucket(BUCKET_NAME).upload_file(Filename = local_file, Key = s3_file)
+        logger.info('Upload successful')
+        delete_excel_file(local_file)
+        return s3_file
+    except FileNotFoundError:
+        logger.info('File was not found')
+        return False
+    except NoCredentialsError:
+        logger.info('Invalid credentials')
+        return False
+
+
 #Delete excel_file from local path
 def delete_excel_file(source_path):
     try:
@@ -365,18 +445,42 @@ def delete_excel_file(source_path):
 
 
 #Generate excel file
-def generate_excel_file(run_id, orgId, integrationId, report_gen_start_time, report_gen_completed_time):
+def generate_excel_file(run_id, orgId, report_gen_start_time, report_gen_completed_time):
     logger.info('Entered into excel generation process')
     excel_data=get_result_by_run(run_id, 'excel-data', {})
     reportname = f"{APP_ID.lower()}" + '-' + datetime.now().strftime('%Y-%m-%d-%I-%M-%S') + '.xlsx'
     filepath = './' + reportname
-    generate_excel(run_id, orgId, integrationId, excel_data, report_gen_start_time, report_gen_completed_time, filepath)
+    generate_excel(run_id, orgId, excel_data, report_gen_start_time, report_gen_completed_time, filepath)
     excel_file_location = f'{PLATFORM_ROUTE}/{run_id}/xls/' + reportname
-    
-    excel_url = upload_excel_s3(filepath, BUCKET_NAME, excel_file_location)
+    if storage_name == 's3':
+        excel_url = upload_excel_s3(filepath, BUCKET_NAME, excel_file_location)
+    elif storage_name == 'ceph':
+        excel_url = upload_excel_ceph(filepath, BUCKET_NAME, excel_file_location)
+    else:
+        logger.info("No storages are found")
     return excel_url
 
 
+def init_mail(org_id, run_id):
+    logger.info('Entered into init_mail method')
+    try:
+        url = BASE_API_URL + f'/analytics/api/v3/tenants/{org_id}/send'
+        data={
+                "runId" : run_id
+            }
+        t1 = int(time.time())
+        res = call_post_requests(url , data=json.dumps(data), verify=False);
+        t2 = int(time.time())
+        duration = t2-t1
+        if duration > API_TIME_STACKS:
+            logging.info('Status update response took %d (greater than %d) seconds', duration, API_TIME_STACKS)
+        logger.info('Status update response is %s', res)
+    except Exception as e:
+        logger.error("Exception raised due to : " + repr(e))
+        traceback.print_exc()
+    logger.info('finished init_mail method')
+
+
 def upload_excel(analysis_run, excel_file):
     timestamp = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
     s3_path = f'{analysis_run.analysis.app.slug}/excel/{timestamp}.xlsx'
